@@ -16,6 +16,7 @@
  */
 package org.apache.logging.log4j.core.appender.db.jdbc;
 
+import java.io.Serializable;
 import java.io.StringReader;
 import java.sql.Clob;
 import java.sql.Connection;
@@ -28,8 +29,11 @@ import java.sql.Types;
 import java.util.ArrayList;
 import java.util.Date;
 import java.util.List;
+import java.util.Objects;
 
+import org.apache.logging.log4j.core.Layout;
 import org.apache.logging.log4j.core.LogEvent;
+import org.apache.logging.log4j.core.StringLayout;
 import org.apache.logging.log4j.core.appender.AppenderLoggingException;
 import org.apache.logging.log4j.core.appender.ManagerFactory;
 import org.apache.logging.log4j.core.appender.db.AbstractDatabaseManager;
@@ -37,8 +41,11 @@ import org.apache.logging.log4j.core.appender.db.ColumnMapping;
 import org.apache.logging.log4j.core.config.plugins.convert.DateTypeConverter;
 import org.apache.logging.log4j.core.config.plugins.convert.TypeConverters;
 import org.apache.logging.log4j.core.util.Closer;
+import org.apache.logging.log4j.message.MapMessage;
 import org.apache.logging.log4j.spi.ThreadContextMap;
 import org.apache.logging.log4j.spi.ThreadContextStack;
+import org.apache.logging.log4j.status.StatusLogger;
+import org.apache.logging.log4j.util.IndexedReadOnlyStringMap;
 import org.apache.logging.log4j.util.ReadOnlyStringMap;
 import org.apache.logging.log4j.util.Strings;
 
@@ -47,6 +54,10 @@ import org.apache.logging.log4j.util.Strings;
  */
 public final class JdbcDatabaseManager extends AbstractDatabaseManager {
 
+    private static StatusLogger logger() {
+        return StatusLogger.getLogger();
+    }
+
     private static final JdbcDatabaseManagerFactory INSTANCE = new JdbcDatabaseManagerFactory();
 
     // NOTE: prepared statements are prepared in this order: column mappings, then column configs
@@ -74,6 +85,7 @@ public final class JdbcDatabaseManager extends AbstractDatabaseManager {
         this.connection = this.connectionSource.getConnection();
         final DatabaseMetaData metaData = this.connection.getMetaData();
         this.isBatchSupported = metaData.supportsBatchUpdates();
+        logger().debug("Closing Connection {}", this.connection);
         Closer.closeSilently(this.connection);
     }
 
@@ -82,6 +94,9 @@ public final class JdbcDatabaseManager extends AbstractDatabaseManager {
         if (this.connection != null || this.statement != null) {
             return this.commitAndClose();
         }
+        if (connectionSource != null) {
+            connectionSource.stop();
+        }
         return true;
     }
 
@@ -90,16 +105,39 @@ public final class JdbcDatabaseManager extends AbstractDatabaseManager {
         try {
             this.connection = this.connectionSource.getConnection();
             this.connection.setAutoCommit(false);
+            logger().debug("Preparing SQL: {}", this.sqlStatement);
             this.statement = this.connection.prepareStatement(this.sqlStatement);
         } catch (final SQLException e) {
             throw new AppenderLoggingException(
-                    "Cannot write logging event or flush buffer; JDBC manager cannot connect to the database.", e
-            );
+                    "Cannot write logging event or flush buffer; JDBC manager cannot connect to the database.", e);
+        }
+    }
+
+    @Deprecated
+    @Override
+    protected void writeInternal(final LogEvent event) {
+        writeInternal(event, null);
+    }
+    
+    private void setFields(final MapMessage<?, ?> mapMessage) throws SQLException {
+        final IndexedReadOnlyStringMap map = mapMessage.getIndexedReadOnlyStringMap();
+        final String simpleName = statement.getClass().getName();
+        int i = 1; // JDBC indices start at 1
+        for (final ColumnMapping mapping : this.columnMappings) {
+            final String source = mapping.getSource();
+            final String key = Strings.isEmpty(source) ? mapping.getName() : source;
+            final Object value = map.getValue(key);
+            if (logger().isTraceEnabled()) {
+                final String valueStr = value instanceof String ? "\"" + value + "\"" : Objects.toString(value, null);
+                logger().trace("{} setObject({}, {}) for key '{}' and mapping '{}'", simpleName, i, valueStr, key,
+                        mapping.getName());
+            }
+            statement.setObject(i++, value);
         }
     }
 
     @Override
-    protected void writeInternal(final LogEvent event) {
+    protected void writeInternal(final LogEvent event, final Serializable serializable) {
         StringReader reader = null;
         try {
             if (!this.isRunning() || this.connection == null || this.connection.isClosed() || this.statement == null
@@ -108,27 +146,35 @@ public final class JdbcDatabaseManager extends AbstractDatabaseManager {
                         "Cannot write logging event; JDBC manager not connected to the database.");
             }
 
-            int i = 1;
+            if (serializable instanceof MapMessage) {
+                setFields((MapMessage<?, ?>) serializable);
+            }
+            int i = 1; // JDBC indices start at 1
             for (final ColumnMapping mapping : this.columnMappings) {
                 if (ThreadContextMap.class.isAssignableFrom(mapping.getType())
-                    || ReadOnlyStringMap.class.isAssignableFrom(mapping.getType())) {
+                        || ReadOnlyStringMap.class.isAssignableFrom(mapping.getType())) {
                     this.statement.setObject(i++, event.getContextData().toMap());
                 } else if (ThreadContextStack.class.isAssignableFrom(mapping.getType())) {
                     this.statement.setObject(i++, event.getContextStack().asList());
                 } else if (Date.class.isAssignableFrom(mapping.getType())) {
-                    this.statement.setObject(i++,
-                        DateTypeConverter.fromMillis(event.getTimeMillis(), mapping.getType().asSubclass(Date.class)));
-                } else if (Clob.class.isAssignableFrom(mapping.getType())) {
-                    this.statement.setClob(i++, new StringReader(mapping.getLayout().toSerializable(event)));
-                } else if (NClob.class.isAssignableFrom(mapping.getType())) {
-                    this.statement.setNClob(i++, new StringReader(mapping.getLayout().toSerializable(event)));
+                    this.statement.setObject(i++, DateTypeConverter.fromMillis(event.getTimeMillis(),
+                            mapping.getType().asSubclass(Date.class)));
                 } else {
-                    final Object value = TypeConverters.convert(mapping.getLayout().toSerializable(event),
-                        mapping.getType(), null);
-                    if (value == null) {
-                        this.statement.setNull(i++, Types.NULL);
-                    } else {
-                        this.statement.setObject(i++, value);
+                    StringLayout layout = mapping.getLayout();
+                    if (layout != null) {
+                        if (Clob.class.isAssignableFrom(mapping.getType())) {
+                            this.statement.setClob(i++, new StringReader(layout.toSerializable(event)));
+                        } else if (NClob.class.isAssignableFrom(mapping.getType())) {
+                            this.statement.setNClob(i++, new StringReader(layout.toSerializable(event)));
+                        } else {
+                            final Object value = TypeConverters.convert(layout.toSerializable(event), mapping.getType(),
+                                    null);
+                            if (value == null) {
+                                this.statement.setNull(i++, Types.NULL);
+                            } else {
+                                this.statement.setObject(i++, value);
+                            }
+                        }
                     }
                 }
             }
@@ -169,14 +215,17 @@ public final class JdbcDatabaseManager extends AbstractDatabaseManager {
         try {
             if (this.connection != null && !this.connection.isClosed()) {
                 if (this.isBatchSupported) {
+                    logger().debug("Executing batch PreparedStatement {}", this.statement);
                     this.statement.executeBatch();
                 }
+                logger().debug("Committing Connection {}", this.connection);
                 this.connection.commit();
             }
         } catch (final SQLException e) {
             throw new AppenderLoggingException("Failed to commit transaction logging event or flushing buffer.", e);
         } finally {
             try {
+                logger().debug("Closing PreparedStatement {}", this.statement);
                 Closer.close(this.statement);
             } catch (final Exception e) {
                 logWarn("Failed to close SQL statement logging event or flushing buffer", e);
@@ -186,6 +235,7 @@ public final class JdbcDatabaseManager extends AbstractDatabaseManager {
             }
 
             try {
+                logger().debug("Closing Connection {}", this.connection);
                 Closer.close(this.connection);
             } catch (final Exception e) {
                 logWarn("Failed to close database connection logging event or flushing buffer", e);
@@ -206,7 +256,7 @@ public final class JdbcDatabaseManager extends AbstractDatabaseManager {
      * @param tableName The name of the database table to insert log events into.
      * @param columnConfigs Configuration information about the log table columns.
      * @return a new or existing JDBC manager as applicable.
-     * @deprecated use {@link #getManager(String, int, ConnectionSource, String, ColumnConfig[], ColumnMapping[])}
+     * @deprecated use {@link #getManager(String, int, Layout, ConnectionSource, String, ColumnConfig[], ColumnMapping[])}
      */
     @Deprecated
     public static JdbcDatabaseManager getJDBCDatabaseManager(final String name, final int bufferSize,
@@ -215,7 +265,7 @@ public final class JdbcDatabaseManager extends AbstractDatabaseManager {
                                                              final ColumnConfig[] columnConfigs) {
 
         return getManager(name,
-            new FactoryData(bufferSize, connectionSource, tableName, columnConfigs, new ColumnMapping[0]),
+            new FactoryData(bufferSize, null, connectionSource, tableName, columnConfigs, new ColumnMapping[0]),
             getFactory());
     }
 
@@ -229,14 +279,39 @@ public final class JdbcDatabaseManager extends AbstractDatabaseManager {
      * @param columnConfigs Configuration information about the log table columns.
      * @param columnMappings column mapping configuration (including type conversion).
      * @return a new or existing JDBC manager as applicable.
+     * @deprecated use {@link #getManager(String, int, Layout, ConnectionSource, String, ColumnConfig[], ColumnMapping[])}
      */
+    @Deprecated
     public static JdbcDatabaseManager getManager(final String name,
                                                  final int bufferSize,
                                                  final ConnectionSource connectionSource,
                                                  final String tableName,
                                                  final ColumnConfig[] columnConfigs,
                                                  final ColumnMapping[] columnMappings) {
-        return getManager(name, new FactoryData(bufferSize, connectionSource, tableName, columnConfigs, columnMappings),
+        return getManager(name, new FactoryData(bufferSize, null, connectionSource, tableName, columnConfigs, columnMappings),
+            getFactory());
+    }
+
+    /**
+     * Creates a JDBC manager for use within the {@link JdbcAppender}, or returns a suitable one if it already exists.
+     *
+     * @param name The name of the manager, which should include connection details and hashed passwords where possible.
+     * @param bufferSize The size of the log event buffer.
+     * @param layout The Appender-level layout
+     * @param connectionSource The source for connections to the database.
+     * @param tableName The name of the database table to insert log events into.
+     * @param columnConfigs Configuration information about the log table columns.
+     * @param columnMappings column mapping configuration (including type conversion).
+     * @return a new or existing JDBC manager as applicable.
+     */
+    public static JdbcDatabaseManager getManager(final String name,
+                                                 final int bufferSize,
+                                                 final Layout<? extends Serializable> layout,
+                                                 final ConnectionSource connectionSource,
+                                                 final String tableName,
+                                                 final ColumnConfig[] columnConfigs,
+                                                 final ColumnMapping[] columnMappings) {
+        return getManager(name, new FactoryData(bufferSize, layout, connectionSource, tableName, columnConfigs, columnMappings),
             getFactory());
     }
 
@@ -253,9 +328,10 @@ public final class JdbcDatabaseManager extends AbstractDatabaseManager {
         private final ColumnConfig[] columnConfigs;
         private final ColumnMapping[] columnMappings;
 
-        protected FactoryData(final int bufferSize, final ConnectionSource connectionSource, final String tableName,
-                              final ColumnConfig[] columnConfigs, final ColumnMapping[] columnMappings) {
-            super(bufferSize);
+        protected FactoryData(final int bufferSize, final Layout<? extends Serializable> layout,
+                final ConnectionSource connectionSource, final String tableName, final ColumnConfig[] columnConfigs,
+                final ColumnMapping[] columnMappings) {
+            super(bufferSize, layout);
             this.connectionSource = connectionSource;
             this.tableName = tableName;
             this.columnConfigs = columnConfigs;
@@ -267,13 +343,19 @@ public final class JdbcDatabaseManager extends AbstractDatabaseManager {
      * Creates managers.
      */
     private static final class JdbcDatabaseManagerFactory implements ManagerFactory<JdbcDatabaseManager, FactoryData> {
+        
+        private static final char PARAMETER_MARKER = '?';
+
         @Override
         public JdbcDatabaseManager createManager(final String name, final FactoryData data) {
             final StringBuilder sb = new StringBuilder("INSERT INTO ").append(data.tableName).append(" (");
             // so this gets a little more complicated now that there are two ways to configure column mappings, but
             // both mappings follow the same exact pattern for the prepared statement
+            int i = 1;
             for (final ColumnMapping mapping : data.columnMappings) {
-                sb.append(mapping.getName()).append(',');
+                final  String mappingName = mapping.getName();
+                logger().trace("Adding INSERT ColumnMapping[{}]: {}={} ", i++, mappingName, mapping);
+                sb.append(mappingName).append(',');
             }
             for (final ColumnConfig config : data.columnConfigs) {
                 sb.append(config.getColumnName()).append(',');
@@ -281,22 +363,32 @@ public final class JdbcDatabaseManager extends AbstractDatabaseManager {
             // at least one of those arrays is guaranteed to be non-empty
             sb.setCharAt(sb.length() - 1, ')');
             sb.append(" VALUES (");
+            i = 1;
             final List<ColumnMapping> columnMappings = new ArrayList<>(data.columnMappings.length);
             for (final ColumnMapping mapping : data.columnMappings) {
+                final String mappingName = mapping.getName();
                 if (Strings.isNotEmpty(mapping.getLiteralValue())) {
+                    logger().trace("Adding INSERT VALUES literal for ColumnMapping[{}]: {}={} ", i, mappingName, mapping.getLiteralValue());
                     sb.append(mapping.getLiteralValue());
+                }
+                if (Strings.isNotEmpty(mapping.getParameter())) {
+                    logger().trace("Adding INSERT VALUES parameter for ColumnMapping[{}]: {}={} ", i, mappingName, mapping.getParameter());
+                    sb.append(mapping.getParameter());
+                    columnMappings.add(mapping);
                 } else {
-                    sb.append('?');
+                    logger().trace("Adding INSERT VALUES parameter marker for ColumnMapping[{}]: {}={} ", i, mappingName, PARAMETER_MARKER);
+                    sb.append(PARAMETER_MARKER);
                     columnMappings.add(mapping);
                 }
                 sb.append(',');
+                i++;
             }
             final List<ColumnConfig> columnConfigs = new ArrayList<>(data.columnConfigs.length);
             for (final ColumnConfig config : data.columnConfigs) {
                 if (Strings.isNotEmpty(config.getLiteralValue())) {
                     sb.append(config.getLiteralValue());
                 } else {
-                    sb.append('?');
+                    sb.append(PARAMETER_MARKER);
                     columnConfigs.add(config);
                 }
                 sb.append(',');
